import numpy as np
class Group:
    def __init__(self, _name):
        self.name = _name
        self.nUpperPrev = 0
        self.stagPoint = None

    def initStructures(self, nPoints):
        self.nPoints = nPoints
        self.nElems = nPoints - 1
        self.nodesCoord = np.zeros((self.nPoints,4))
        self.elemsCoord = np.zeros((self.nPoints,4))
        
        self.V = np.zeros((self.nPoints,3))
        self.M = np.zeros(self.nPoints)
        self.Rho = np.zeros(self.nPoints)
        self.blowingVel = np.zeros(self.nElems)

        self.deltaStarExt = np.zeros(self.nPoints)
        self.xxExt = np.zeros(self.nPoints)

    def updateVars(self, nodes, _V, _M, _Rho):
        self.nodesCoord = nodes
        self.nPoints = len(self.nodesCoord[:,0])
        self.nElems = self.nPoints - 1
        self.V = _V
        self.M = _M
        self.Rho = _Rho
        self.computeStagPoint()

    def getXUpper(self):
        return np.flip(self.nodesCoord[:self.stagPoint+1,0])
    def getXLower(self):
        return self.nodesCoord[self.stagPoint:, 0]
    def getYUpper(self):
        return np.flip(self.nodesCoord[:self.stagPoint+1,1])
    def getYLower(self):
        return self.nodesCoord[self.stagPoint:, 1]
    def getZUpper(self):
        return np.flip(self.nodesCoord[:self.stagPoint+1,2])
    def getZLower(self):
        return self.nodesCoord[self.stagPoint:, 2]
    
    def getVelocityXUpper(self):
        return np.flip(self.V[:self.stagPoint+1,0])
    def getVelocityXLower(self):
        return self.V[self.stagPoint:, 0]
    def getVelocityYUpper(self):
        return np.flip(self.V[:self.stagPoint+1,1])
    def getVelocityYLower(self):
        return self.V[self.stagPoint:, 1]
    def getVelocityZUpper(self):
        return np.flip(self.V[:self.stagPoint+1,2])
    def getVelocityZLower(self):
        return self.V[self.stagPoint:, 2]

    def getMachUpper(self):
        return np.flip(self.M[:self.stagPoint+1])
    def getMachLower(self):
        return self.M[self.stagPoint:]

    def getRhoUpper(self):
        return np.flip(self.Rho[:self.stagPoint+1])
    def getRhoLower(self):
        return self.Rho[self.stagPoint:]

    def setBlowingVelocity(self, blwVel):
        self.blowingVel = blwVel
    def setConnectList(self, _connectListNodes, _connectListElems):
        if self.name != 'iWing' and self.name != 'iWake':
            raise RuntimeError('Cannot set connectivity lists for viscous structures', self.name)
        self.connectListNodes = _connectListNodes
        self.connectListElems = _connectListElems
    def connectBlowingVel(self):
        if self.name != 'iWing' and self.name != 'iWake':
            raise RuntimeWarning('Can not connect blowing velocities for viscous structure. Maybe it was not set.', self.name)
        self.blowingVel = self.blowingVel[self.connectListElems.argsort()]

    def isMeshChanged(self):
        return self.nPoints - self.nUpperPrev
    def updatePrev(self):
        self.nUpperPrev = self.nPoints
    def computeStagPoint(self):
        if 'iWake' in self.name or 'vWake' in self.name:
            self.stagPoint = None
        else:
            if self.stagPoint is None:
                self.stagPoint = np.argmin(np.sqrt(self.V[:,0]**2 + self.V[:,1]**2))