

import numpy as np
from scipy.interpolate import RBFInterpolator
class RbfInterpolator:
    def __init__(self, _cfg):
        self.cfg = _cfg

    def inviscidToViscous(self, iDict, vDict, couplIter):
        ## Airfoil
        # Find stagnation point
        for iSec in range(len(vDict)):
            for iReg, reg in enumerate(vDict[iSec]):
                v = np.zeros((reg.nodesCoord.shape[0], 3))
                M = np.zeros(reg.nodesCoord.shape[0])
                rho = np.zeros(reg.nodesCoord.shape[0])
                for iDim in range(3):
                    v[:,iDim] = self.__rbfToSection(iDict[iReg].nodesCoord[:,:(self.cfg['nDim'] if iDict[iReg].name == 'iWing' else 1)], iDict[iReg].V[:,iDim], reg.nodesCoord[:,:(self.cfg['nDim'] if 'vAirfoil' in reg.name else 1)])
                M = self.__rbfToSection(iDict[iReg].nodesCoord[:,:(self.cfg['nDim'] if iDict[iReg].name == 'iWing' else 1)], iDict[iReg].M, reg.nodesCoord[:,:(self.cfg['nDim'] if 'vAirfoil' in reg.name else 1)])
                rho = self.__rbfToSection(iDict[iReg].nodesCoord[:,:(self.cfg['nDim'] if iDict[iReg].name == 'iWing' else 1)], iDict[iReg].Rho, reg.nodesCoord[:,:(self.cfg['nDim'] if 'vAirfoil' in reg.name else 1)])
                vDict[iSec][iReg].updateVars(vDict[iSec][iReg].nodesCoord, v, M, rho)

    def viscousToInviscid(self, iDict, vDict):
        if self.cfg['nDim'] == 2:
            for iReg, reg in enumerate(iDict):
                iDict[iReg].blowingVel = self.__rbfToSection(vDict[0][iReg].elemsCoordTr[:,:(self.cfg['nDim']-1 if 'vAirfoil' in reg.name else 1)], vDict[0][iReg].blowingVel, reg.elemsCoordTr[:,:(self.cfg['nDim']-1 if reg.name == 'iWing' else 1)])
        elif self.cfg['nDim'] == 3:
            for iReg, reg in enumerate(iDict):
                viscElemsCoord = np.zeros((0,3))
                viscBlowing = np.zeros(0)
                for iSec, sec in enumerate(vDict):
                    viscElemsCoord = np.row_stack((viscElemsCoord, sec[iReg].elemsCoord[:,:3]))
                    viscBlowing = np.concatenate((viscBlowing, sec[iReg].blowingVel))
                if 'Sym' in self.cfg:
                    for s in self.cfg['Sym']:
                        dummy = viscElemsCoord.copy()
                        dummy[:,2] = (dummy[:,2] - s)*(-1) + s
                        viscElemsCoord = np.row_stack((viscElemsCoord, dummy))
                        viscBlowing = np.concatenate((viscBlowing, viscBlowing))
                reg.blowingVel = self.__rbfToSection(viscElemsCoord, viscBlowing, reg.elemsCoord[:,:3])
        else:
            raise RuntimeError('Incorrect number of dimensions', self.cfg['nDim'])

    def __rbfToSection(self, x, var, xs):
        if np.all(var == var[0]):
            varOut = RBFInterpolator(x, var, neighbors=1, kernel='linear', smoothing=0.0, degree=0)(xs)
        else:
            varOut = RBFInterpolator(x, var, neighbors=self.cfg['neighbors'], kernel=self.cfg['rbftype'], smoothing=self.cfg['smoothing'], degree=self.cfg['degree'])(xs)
        return varOut